//            Copyright Daniel Trebbien 2010.
// Distributed under the Boost Software License, Version 1.0.
//   (See accompanying file LICENSE_1_0.txt or the copy at
//         http://www.boost.org/LICENSE_1_0.txt)

#include <cstddef>
#include <iostream>
#include <map>
#include <boost/graph/adjacency_list.hpp>
#include <boost/graph/stoer_wagner_mincut.hpp>
#include <boost/property_map/property_map.hpp>

struct edge_t
{
    unsigned long first;
    unsigned long second;
};

int main()
{
    using namespace std;

    typedef boost::adjacency_list<boost::vecS, boost::vecS, boost::undirectedS, boost::no_property, boost::property<boost::edge_weight_t, int> > undirected_graph;
    typedef boost::graph_traits<undirected_graph>::vertex_descriptor vertex_descriptor;
    typedef boost::property_map<undirected_graph, boost::edge_weight_t>::type weight_map_type;
    typedef boost::property_traits<weight_map_type>::value_type weight_type;
    
    edge_t edges[] = {{3, 4}, {3, 6}, {3, 5}, {0, 4}, {0, 1}, {0, 6}, {0, 7},
        {0, 5}, {0, 2}, {4, 1}, {1, 6}, {1, 5}, {6, 7}, {7, 5}, {5, 2}, {3, 4}};
    weight_type ws[] = {0, 3, 1, 3, 1, 2, 6, 1, 8, 1, 1, 80, 2, 1, 1, 4};
    undirected_graph g(edges, edges + 16, ws, 8, 16);
    
    map<vertex_descriptor, bool> parity;
    boost::associative_property_map<map<vertex_descriptor, bool> > parities(parity);
    int w = boost::stoer_wagner_mincut(g, get(boost::edge_weight, g), boost::parity_map(parities));
    
    cout << "The min-cut weight of G is " << w << ".\n" << endl;
    
    cout << "One set of vertices consists of:" << endl;
    map<vertex_descriptor, bool>::iterator it = parity.begin(), end = parity.end();
    for (; it != end; ++it) {
        if (it->second)
            cout << it->first << endl;
    }
    cout << endl;
    
    cout << "The other set of vertices consists of:" << endl;
    for (it = parity.begin(); it != end; ++it) {
        if (!it->second)
            cout << it->first << endl;
    }
    cout << endl;
    
    return EXIT_SUCCESS;
}
